function [mLARS]=LARS(x0,y0)                                                %the LARS algorithm
    [ss,ncov]=size(x0);                                                     %the data size
    x=x0;y=y0;                                                              %the working data
    y=(y-mean(y))/std(y);                                                   %standardize y
    for j=1:ncov; x(:,j)=(x(:,j)-mean(x(:,j)))/std(x(:,j)); end             %standardize x
    rho=abs(y'*x)/ss;m=find(rho==max(rho));                                 %find 1st direction
    mLARS=[m];Model=[1:ncov];Model(m)=[];                                   %initiate 1st lars model

    nStep=min(50,ncov)-1;B=[];beta=zeros(ncov,1);sigma2=[];                 %before the main loop
    for k=1:nStep                                                           %do every LARS step
        sigma2=[sigma2,mean(y.^2)];                                         %the \hat\sigma^2
        mSize=length(mLARS);                                                %the current model size
        xA=x(:,mLARS);xB=x(:,Model);                                        %included and non-included X-var.
        rho=y'*xA;                                                          %the equal correlation
        mysign=diag(sign(y'*xA));xA=xA*mysign;                              %correct the X-direction
        UA=xA*inv(xA'*xA)*ones(mSize,1);                                    %equi-angular direciton
        c=xB'*y;                                                            %corr(non-included-X-var,response)
        a=xB'*UA;                                                           %corr(non-included-X-var,equi-angular)
        Cmax=max(abs(rho));                                                 %the maximal equi-angular corr
        g1=(Cmax-c)./(1-a);g1=g1.*(g1>0)+max(g1).*(g1<=0);                  %if Cmax==+c_j
        g2=(Cmax+c)./(1+a);g2=g2.*(g2>0)+max(g2).*(g2<=0);                  %if Cmax==-c_j
        g=g1*(min(g1)<min(g2))+g2*(min(g2)<=min(g1));gamma=min(g);          %find gamma
        m=find(g==min(g));m=m(1);                                           %find next active variable
        y=y-gamma*UA;                                                       %update residual
        beta(mLARS)=beta(mLARS)+gamma*mysign*inv(xA'*xA)*ones(mSize,1);     %update beta coeff
        mLARS=[mLARS,Model(m)];Model(m)=[];                                 %update lars models
    end                                             
    BIC=log(sigma2)+[1:nStep]*(log(ss)+2*log(ncov))/ss;                     %Chen and Chen's BIC score
    pos=find(BIC==min(BIC));pos=pos(1)-1;                                   %position of min(BIC)
    mLARS=mLARS(1:pos);                                                     %final mLARS model