function [R2,x0,y0,x1,y1,beta0,betapair0,Beta0,pairindex,ss0,ncov,meanx,varx,meany]=simulator4(model,opt)     
 
%
% input
% 
% two parameters: "model" and "option", control different simulation settings.
%"model" controls the correlation structure
%"option" controls the signal-to-noise ratio


%
% output
%
    %R2 is the noise standard deviation
    % (x0, y0) is the training set 
    % August 2014, return raw data only
    % (x1, y1) is the test set (raw data also)
    % beta0 is the coefficient vector of main effects (including 0's; no intercept)
    % betapair0 is the coefficient vector of important interactions (not including 0's)
    % Beta0 is the coefficient vector of main and 2-way interations 
    % (no intercept, including all 0's), saved in the sparse mode
    % pairindex specifies the parent locations of important interaction terms
    % ss0 is the training data sample size 
    % ncov is the data dimension
    % meanx is the column means of x0
    % varx is the column variance of x0
    % meany is the mean of y0
    
    MyR2=[2,3,4];
    switch model

          case 1                                                          
            R2=MyR2(opt);
            ss=200;ncov=500;ss0=ss/2;                                    %select ss and ncov
            x=randn(ss,ncov);                                           %generate stand. norm.
            XX = genXX(x);                                           %generate bigX main effects and all two-way interaction
            ncov0 = 4;                                                  %number of important main effects 
            beta0 = [3;0;3;0;0;3;0;0;0;3;zeros(ncov-ncov0-6,1)];						%generate beta0
            pairindex = [1 3; 1 6; 3 10; 6 10];							%important pair locations
            betapair0 = 5*[0.4; 0.4; 0.4; 0.4];							%important pair coefficients
          
          
        case 2                                                          %auto-regressive
            R2=MyR2(opt);                                               %the R-Square
            ss=200;ncov=500;ss0=ss/2;                                    %select ss and ncov
            rho=0.5;                                                       %correlation coeff.
            x=randn(ss,ncov);                                           %generate X
            for j=2:ncov;
				x(:,j)=rho*x(:,j-1)+sqrt(1-rho^2)*x(:,j);
            end                                                         %generate X
            XX = genXX(x);          
            ncov0 = 4;                                                 %number of important main effects 
            beta0 = [3;0;3;0;0;3;0;0;0;3;zeros(ncov-ncov0-6,1)];          %generate beta0
            pairindex = [1 3;1 6;3 10;6 10];    %important pair locations
            betapair0 = 2*ones(4,1);              %important pair coefficients
           
                 
       case 3                                                          %auto-regressive, p = 50
            R2=MyR2(opt);                                               %the R-Square
            ss=800;ncov=5000;ss0=ss/2;                                    %select ss and ncov
            rho=0.5;                                                       %correlation coeff.
            x=randn(ss,ncov);                                           
            for j=2:ncov;
			x(:,j)=rho*x(:,j-1)+sqrt(1-rho^2)*x(:,j);
            end									
            ncov0 = 10;                                                 %number of important main effects 
            beta0 = [3*ones(5,1);2*ones(5,1);zeros(ncov-ncov0,1)];          %generate beta0
            beta0 = sparse(beta0);
            pairindex = [1 2;1 3;2 5;2 3;3 4;6 8;6 10;7 8;7 9;9 10];                           %important pair locations
            betapair0 = [2*ones(5,1);ones(5,1)];                           %important pair coefficients
     
    
      case 4                                                          %auto-regressive, p = 50
            R2=MyR2(opt);                                               %the R-Square
            ss=800;ncov=10000;ss0=ss/2;                                    %select ss and ncov
            rho=0.5;                                                       %correlation coeff.
            x=randn(ss,ncov);                                           
            for j=2:ncov;
			x(:,j)=rho*x(:,j-1)+sqrt(1-rho^2)*x(:,j);
            end									
            ncov0 = 10;                                                 %number of important main effects 
            beta0 = [3*ones(5,1);2*ones(5,1);zeros(ncov-ncov0,1)];          %generate beta0
            beta0 = sparse(beta0);
            pairindex = [1 2;1 3;2 5;2 3;3 4;6 8;6 10;7 8;7 9;9 10];                           %important pair locations
            betapair0 = [2*ones(5,1);ones(5,1)];          
    

        case 5                                                          %cs, p=5000
            R2=MyR2(opt);                                               %the R-Square
            ss=800;ncov=5000;ss0=ss/2;                                    %select ss and ncov
            w = randn(ss,1);                                                       %correlation coeff.
            x=randn(ss,ncov);                                           
            for j=1:ncov;
			x(:,j)=(x(:,j)+w)/sqrt(2);
            end									
            ncov0 = 10;                                                 %number of important main effects 
            beta0 = [3*ones(5,1);2*ones(5,1);zeros(ncov-ncov0,1)];          %generate beta0
            beta0 = sparse(beta0);
            pairindex = [1 2;1 3;2 5;2 3;3 4;6 8;6 10;7 8;7 9;9 10];                           %important pair locations
            betapair0 = [2*ones(5,1);ones(5,1)];                           %important pair coefficients
     
    
      case 6                                                          %auto-regressive, p = 50
            R2=MyR2(opt);                                               %the R-Square
            ss=800;ncov=5000;ss0=ss/2;                                    %select ss and ncov
            z = randn(ss,ncov);
            w = randn(ss,ncov);
            ncov0 = 10;
            zz = zeros(ss,1);
            for j=1:ncov0;
    			x(:,j)=(z(:,j)+w(:,j))/sqrt(2);
                zz = zz+z(:,j);
            end
            for j = (1+ncov0):ncov
                x(:,j) = (z(:,j)+zz)/2;
            end    
            beta0 = [3*ones(5,1);2*ones(5,1);zeros(ncov-ncov0,1)];          %generate beta0
            beta0 = sparse(beta0);
            pairindex = [1 2;1 3;2 5;2 3;3 4;6 8;6 10;7 8;7 9;9 10];                           %important pair locations
            betapair0 = [2*ones(5,1);ones(5,1)];          
    

       case 7                                                          %cs, p = 10000
            R2=MyR2(opt);                                               %the R-Square
            ss=800;ncov=10000;ss0=ss/2;                                    %select ss and ncov
            w = randn(ss,1);                                                       %correlation coeff.
            x=randn(ss,ncov);                                           
            for j=1:ncov;
			x(:,j)=(x(:,j)+w)/sqrt(2);
            end											
            ncov0 = 10;                                                 %number of important main effects 
            beta0 = [3*ones(5,1);2*ones(5,1);zeros(ncov-ncov0,1)];          %generate beta0
            beta0 = sparse(beta0);
            pairindex = [1 2;1 3;2 5;2 3;3 4;6 8;6 10;7 8;7 9;9 10];                           %important pair locations
            betapair0 = [2*ones(5,1);ones(5,1)];                           %important pair coefficients
     
    
      case 8                                                          %challenging, p = 10000
            R2=MyR2(opt);                                               %the R-Square
            ss=800;ncov=10000;ss0=ss/2;                                    %select ss and ncov
            z = randn(ss,ncov);
            w = randn(ss,ncov);
            ncov0 = 10;
            zz = zeros(ss,1);
            for j=1:ncov0;
    			x(:,j)=(z(:,j)+w(:,j))/sqrt(2);
                zz = zz+z(:,j);
            end
            for j = (1+ncov0):ncov
                x(:,j) = (z(:,j)+zz)/2;
            end    
                                                      %number of important main effects 
            beta0 = [3*ones(5,1);2*ones(5,1);zeros(ncov-ncov0,1)];          %generate beta0
            beta0 = sparse(beta0);
            pairindex = [1 2;1 3;2 5;2 3;3 4;6 8;6 10;7 8;7 9;9 10];                           %important pair locations
            betapair0 = [2*ones(5,1);ones(5,1)];          
    

        case 9                                                          %weak, auto-regressive, p = 5000
            R2=MyR2(opt);                                               %the R-Square
            ss=800;ncov=5000;ss0=ss/2;                                    %select ss and ncov
            rho=0.5;                                                       %correlation coeff.
            x=randn(ss,ncov);                                           
            for j=2:ncov;
			x(:,j)=rho*x(:,j-1)+sqrt(1-rho^2)*x(:,j);
            end									
            ncov0 = 10;                                                 %number of important main effects 
            beta0 = [3*ones(5,1);2*ones(5,1);zeros(ncov-ncov0,1)];          %generate beta0
            beta0 = sparse(beta0);
            pairindex = [1 2;1 13;2 15;2 3;3 4;6 18;6 10;7 18;7 9;19 10];                           %important pair locations
            betapair0 = [2*ones(5,1);ones(5,1)];                           %important pair coefficients
     
    
      case 10                                                          %weak, cs, p = 5000
            R2=MyR2(opt);                                               %the R-Square
            ss=800;ncov=5000;ss0=ss/2;                                    %select ss and ncov
           	w = randn(ss,1);                                                       %correlation coeff.
            x=randn(ss,ncov);                                           
            for j=1:ncov;
			x(:,j)=(x(:,j)+w)/sqrt(2);
            end							
            ncov0 = 10;                                                 %number of important main effects 
            beta0 = [3*ones(5,1);2*ones(5,1);zeros(ncov-ncov0,1)];          %generate beta0
            beta0 = sparse(beta0);
            pairindex = [1 2;1 13;2 15;2 3;3 4;6 18;6 10;7 18;7 9;19 10];                           %important pair locations
            betapair0 = [2*ones(5,1);ones(5,1)];          
    

        case 11                                                          %weak challenging, p = 5000
            R2=MyR2(opt);                                               %the R-Square
            ss=800;ncov=5000;ss0=ss/2;                                    %select ss and ncov
             z = randn(ss,ncov);
            w = randn(ss,ncov);
            ncov0 = 10;
            zz = zeros(ss,1);
            for j=1:ncov0;
    			x(:,j)=(z(:,j)+w(:,j))/sqrt(2);
                zz = zz+z(:,j);
            end
            for j = (1+ncov0):ncov
                x(:,j) = (z(:,j)+zz)/2;
            end    
                                                                         %number of important main effects 
            beta0 = [3*ones(5,1);2*ones(5,1);zeros(ncov-ncov0,1)];          %generate beta0
            beta0 = sparse(beta0);
            pairindex = [1 2;1 13;2 15;2 3;3 4;6 18;6 10;7 18;7 9;19 10];  
                   %important pair locations
            betapair0 = [2*ones(5,1);ones(5,1)];                           %important pair coefficients
            
        case 12                                                          %cs, p=5000
            R2=MyR2(opt);                                               %the R-Square
            ss=800;ncov=5000;ss0=ss/2;                                    %select ss and ncov
            w = randn(ss,1);                                                       %correlation coeff.
            x=randn(ss,ncov);
            ncov0 =10;
            zz = zeros(ss,1);
            for j=1:ncov0;
                zz = zz+x(:,j)*(-1)^(j+1)/5;
            end
            for j=(ncov-49):ncov;
                x(:,j)=zz+sqrt(25-ncov0)*x(:,j)/5;
            end									
                                                             %number of important main effects 
            beta0 = [3*ones(5,1);2*ones(5,1);zeros(ncov-ncov0,1)];          %generate beta0
            beta0 = sparse(beta0);
            pairindex = [1 2;1 3;2 5;2 3;3 4;6 8;6 10;7 8;7 9;9 10];                           %important pair locations
            betapair0 = [2*ones(5,1);ones(5,1)];     
            
            
        case 13                                                          %cs, p=5000
            R2=MyR2(opt);                                               %the R-Square
            ss=800;ncov=10000;ss0=ss/2;                                    %select ss and ncov
            w = randn(ss,1);                                                       %correlation coeff.
            x=randn(ss,ncov);
            ncov0 =10;
            zz = zeros(ss,1);
            for j=1:ncov0;
                zz = zz+x(:,j)*(-1)^(j+1)/5;
            end
            for j=(ncov-49):ncov;
                x(:,j)=zz+sqrt(25-ncov0)*x(:,j)/5;
            end									
                                                             %number of important main effects 
            beta0 = [3*ones(5,1);2*ones(5,1);zeros(ncov-ncov0,1)];          %generate beta0
            beta0 = sparse(beta0);
            pairindex = [1 2;1 3;2 5;2 3;3 4;6 8;6 10;7 8;7 9;9 10];                           %important pair locations
            betapair0 = [2*ones(5,1);ones(5,1)];         
            
            
            
    end


	Dim = ncov*(ncov+3)/2;
	npair0 = length(betapair0);
	y=x*beta0;                                                  
		%get conditional mean for main effects
	Beta0 = sparse(zeros(Dim,1));                                       
		%generate long Beta0 for all main and interaction models
	Beta0(1:ncov)=beta0;
	for k=1:npair0
		intid = findid(pairindex(k,1),pairindex(k,2),ncov);
		Beta0(intid) = betapair0(k);
		y = y+betapair0(k)*x(:,pairindex(k,1)).*x(:,pairindex(k,2));
	end; 
    Beta0 = sparse(Beta0);
    y=y+R2*randn(ss,1);                                     
    
    %now, we standardize the data
    xtrain=x(1:ss0,:);
    ytrain=y(1:ss0); 
    meanx = mean(xtrain,1); %only training data
    varx =var(xtrain,1);
    %for jj=1:ncov
    %    for kk=jj:ncov
    %        tmp = xtrain(:,jj).*xtrain(:,kk);
    %        meanXX = [meanXX mean(tmp)]; 
    %        varXX = [varXX var(tmp)];
    %    end
    %end
    meany =mean(ytrain);
    
    x0 = xtrain;
    y0 = ytrain;
    %August 2014, return raw data only
    %standardize training
    %for j=1:ncov
	%	x0(:,j) = (x0(:,j)-meanx(j))./sqrt(varx(j));
    %end
    %y0 = ytrain - meany;
    %the training standardized-data
    
    %test data (raw)
    x1= x((ss0+1):ss,:);
    y1= y((ss0+1):ss);                                                      %the testing raw-data
    
